import { Component } from 'yuzu';
import { IComponentConstructable } from 'yuzu/types';
export interface ILoadableOptions<C> {
    component: IComponentConstructable<C>;
    loader?: IComponentConstructable<Component<any, any>> | null;
    template?: (...args: any[]) => string | void;
    renderRoot?: string | (($el: Element) => Element);
    fetchData: (ctx: Component<any, any>) => Record<string, any> | void;
    options?: Record<string, any>;
    props?: Record<string, any> | ((props: Record<string, any>) => Record<string, any>);
}
export interface ILoadableState {
    props: Record<string, any>;
}
/**
 * A function that returns a configurable async component loader.
 *
 * @param {object} opts
 * @param {Component} opts.component Component to initialize when `opts.fetchData` is resolved
 * @param {function} [opts.fetchData] A function to load remote data. Must return a promise
 * @param {function} [opts.template] Component template. A function returning a string
 * @param {Component} [opts.loader] Loader component. Shown during `opts.fetchData` execution
 * @param {string|function} [opts.renderRoot='div'] Tag used for the element holding the async component. Either a string or a function returning a DOM element.
 * @param {object} [opts.options] Component options
 * @param {props} [opts.props] Computed state attached to the component
 * @returns {LoadableComponent} Component constructor
 * @example
 *
 * import { Component } from 'yuzu';
 * import { Loadable } from 'yuzu-loadable';
 *
 * class Message extends Component {}
 * class Loader extends Component {}
 *
 * const delay = () => new Promise((resolve) => {
 *   setTimeout(() => {
 *     resolve({ message: 'Hello World' });
 *   }, 1000);
 * });
 *
 * const template = (props) => `<div class="Message">${props.message}</div>`;
 *
 * const LoadableMessage = Loadable({
 *   component: Message,
 *   loader: Loader,
 *   fetchData: delay,
 *   template
 * });
 *
 * const message = new LoadableMessage().mount('#loadable-message');
 */
export declare function Loadable<C extends Component<any, any> = Component<any, any>>(opts: ILoadableOptions<C>): IComponentConstructable<Component<ILoadableState, ILoadableOptions<C>>>;
