import invariant from 'tiny-invariant';
import { noop, isElement, evaluate } from 'yuzu-utils';
import { Component } from 'yuzu';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

// A type of promise-like that resolves synchronously and supports only one observer
var _iteratorSymbol =
/*#__PURE__*/
typeof Symbol !== "undefined" ? Symbol.iterator || (Symbol.iterator =
/*#__PURE__*/
Symbol("Symbol.iterator")) : "@@iterator"; // Asynchronously iterate through an object's values
var _asyncIteratorSymbol =
/*#__PURE__*/
typeof Symbol !== "undefined" ? Symbol.asyncIterator || (Symbol.asyncIterator =
/*#__PURE__*/
Symbol("Symbol.asyncIterator")) : "@@asyncIterator"; // Asynchronously iterate on a value using it's async iterator if present, or its synchronous iterator if missing

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
} // Asynchronously await a promise and pass the result to a finally continuation

/**
 * A function that returns a configurable async component loader.
 *
 * @param {object} opts
 * @param {Component} opts.component Component to initialize when `opts.fetchData` is resolved
 * @param {function} [opts.fetchData] A function to load remote data. Must return a promise
 * @param {function} [opts.template] Component template. A function returning a string
 * @param {Component} [opts.loader] Loader component. Shown during `opts.fetchData` execution
 * @param {string|function} [opts.renderRoot='div'] Tag used for the element holding the async component. Either a string or a function returning a DOM element.
 * @param {object} [opts.options] Component options
 * @param {props} [opts.props] Computed state attached to the component
 * @returns {LoadableComponent} Component constructor
 * @example
 *
 * import { Component } from 'yuzu';
 * import { Loadable } from 'yuzu-loadable';
 *
 * class Message extends Component {}
 * class Loader extends Component {}
 *
 * const delay = () => new Promise((resolve) => {
 *   setTimeout(() => {
 *     resolve({ message: 'Hello World' });
 *   }, 1000);
 * });
 *
 * const template = (props) => `<div class="Message">${props.message}</div>`;
 *
 * const LoadableMessage = Loadable({
 *   component: Message,
 *   loader: Loader,
 *   fetchData: delay,
 *   template
 * });
 *
 * const message = new LoadableMessage().mount('#loadable-message');
 */

function Loadable(opts) {
  var _a;

  var Child = opts.component,
      params = _objectWithoutPropertiesLoose(opts, ["component"]);

  var childName = Child.displayName || Child.name || '';
  /**
   * Async component loader
   *
   * @class
   * @extends Component
   * @param {object} [config]
   * @param {Component} config.component Component to initialize when `config.fetchData` is resolved
   * @param {function} [config.fetchData] A function to load remote data. Must return a promise
   * @param {function} [config.template] Component template. A function returning a string
   * @param {Component} [config.loader] Loader component. Shown during `config.fetchData` execution
   * @param {string|function} [config.renderRoot='div'] Tag used for the element holding the async component. Either a string or a function returning a DOM element.
   * @param {object} [config.options] Component options
   * @param {object|function} [config.props] Computed state attached to the component
   * @returns {LoadableComponent}
   */

  var LoadableComponent = (_a =
  /*#__PURE__*/
  function (_Component) {
    _inheritsLoose(_a, _Component);

    function _a() {
      var _this;

      _this = _Component.apply(this, arguments) || this;
      _this.state = {
        props: {}
      };
      return _this;
    }

    var _proto = _a.prototype;

    _proto.defaultOptions = function defaultOptions() {
      return Object.assign({
        fetchData: noop,
        component: Child,
        template: noop,
        loader: null,
        options: {},
        renderRoot: 'div',
        props: function props(v) {
          return v;
        }
      }, params);
    }
    /**
     * Initialize hook.
     *
     * Will replace the current root element contents with an empty element used as root for both the optional loader (passed as `config.loader` to the constructor)
     * the rendered component root.
     *
     * It will then set the loader (if available) and fetch the data (`config.fetchData`) before initializing the async component.
     *
     * @async
     * @memberof LoadableComponent
     * @returns {LoadableComponent}
     */
    ;

    _proto.initialize = function initialize() {
      try {
        var _this3 = this;

        var _this3$options = _this3.options,
            fetchData = _this3$options.fetchData,
            renderRoot = _this3$options.renderRoot;
        var $el = _this3.$el; // empty the component

        $el.textContent = '';
        !(typeof renderRoot === 'function' || typeof renderRoot === 'string') ? process.env.NODE_ENV !== "production" ? invariant(false, '"options.renderRoot" must be either a function or a string') : invariant(false) : void 0;
        var async = typeof renderRoot === 'string' ? document.createElement(renderRoot) : renderRoot && renderRoot($el);

        if (async) {
          _this3.$els.async = $el.appendChild(async);
        }

        return Promise.resolve(_this3.setLoader()).then(function () {
          return _catch(function () {
            return Promise.resolve(fetchData(_this3)).then(function (data) {
              if (data) {
                _this3.setState({
                  props: data
                });
              }

              return Promise.resolve(_this3.render()).then(function (root) {
                return Promise.resolve(_this3.setComponent(root)).then(function () {
                  return _this3;
                });
              });
            });
          }, function (e) {
            console.error(e);
            return _this3;
          });
        });
      } catch (e) {
        return Promise.reject(e);
      }
    }
    /**
     * Initializes the rendered component as child of the loadable instance.
     *
     * The component will be attached as child into `$refs.async`.
     *
     * @memberof LoadableComponent
     * @param {Element} [root] Component root element. Will default to `$els.async` if not defined.
     * @return {Promise}
     * @example
     * class Loader extends Component {
     *   // ...
     * }
     *
     * const LoadableMessage = Loadable({
     *   component: Message,
     * });
     *
     * const loadable = new LoadableMessage();
     * loadable.setComponent()
     *
     * instanceof loadable.$refs.async === Message
     */
    ;

    _proto.setComponent = function setComponent(root) {
      var _this$options = this.options,
          component = _this$options.component,
          options = _this$options.options,
          props = _this$options.props;
      return this.setRef(_extends({
        el: isElement(root) ? root : this.$els.async,
        id: 'async',
        component: component
      }, options), evaluate(props, this.state.props));
    }
    /**
     * Initializes a loader.
     *
     * If defined, will initialize the loader component set in `config.loader`,
     * else will return a resolved promise.
     *
     * The loader will be attached as child into `$refs.async`.
     *
     * @memberof LoadableComponent
     * @returns {Promise}
     * @example
     *
     * class Loader extends Component {
     *   // ...
     * }
     *
     * const LoadableMessage = Loadable({
     *   component: Message,
     *   loader: Loader
     * });
     *
     * const loadable = new LoadableMessage();
     * loadable.setLoader()
     *
     * instanceof loadable.$refs.async === Loader
     */
    ;

    _proto.setLoader = function setLoader() {
      try {
        var _exit2 = false;

        var _this5 = this;

        var loader = _this5.options.loader;
        return Promise.resolve(function () {
          if (loader) {
            _exit2 = true;
            return Promise.resolve(_this5.setRef({
              id: 'async',
              el: _this5.$els.async,
              component: loader
            }));
          }
        }());
      } catch (e) {
        return Promise.reject(e);
      }
    }
    /**
     * Renders an optional template.
     *
     * Executes the template function set in `config.template` passing an object containing a `props`
     * key equal to the `state.props` key.
     *
     * @memberof LoadableComponent
     * @returns {string|null} returns null if the template function returns a falsy value
     * @example
     * const template = (props) => `<div>${props.message}</div>`;
     *
     * const LoadableMessage = Loadable({
     *   component: Message,
     *   template
     * });
     *
     * const loadable = new LoadableMessage();
     * loadable.setState({ props: { message: 'Hello World' }});
     * const html = loadable.render();
     * html === '<div>Hello World</div>';
     */
    ;

    _proto.render = function render() {
      var template = this.options.template;
      var props = this.state.props;
      var wrapper = document.createElement('div');
      var html = template && template({
        props: props,
        options: this.options
      });

      if (html) {
        wrapper.innerHTML = html;

        if (process.env.NODE_ENV !== 'production') {
          if (wrapper.childElementCount > 1) {
            this.$warn('Multi-root templates are not supported. Just the first root element will be rendered.');
          }
        }

        return wrapper.firstElementChild;
      }

      return null;
    };

    return _a;
  }(Component), _a.root = "[data-loadable=\"" + childName + "\"], [data-loadable][data-component=\"" + childName + "\"]", _a);
  Object.defineProperty(LoadableComponent, 'displayName', {
    value: "Loadable" + (childName || 'Component')
  });
  return LoadableComponent;
}

export { Loadable };
//# sourceMappingURL=yuzu-loadable.esm.js.map
